#include <windows.h>
1	VERSIONINFO
FILEVERSION 0,1,0,0
PRODUCTVERSION 0,1,0,0
FILEOS VOS_NT
FILETYPE VFT_APP
{
BLOCK "StringFileInfo"
{
	BLOCK "040904b0"
	{
		VALUE "CompanyName", "The icoextract authors"
		VALUE "FileDescription", "icoextract Test Application"
		VALUE "FileVersion", "0.1.0"
		VALUE "InternalName", "testapp"
		VALUE "ProductName", "icoextract"
		VALUE "OriginalFilename", "testapp.exe"
		VALUE "ProductVersion", "0.1.0"
	}
}

BLOCK "VarFileInfo"
{
	VALUE "Translation", 0x0409, 1252
}
}
